/* Filling a vector with thread identification.
 *
 * This sample is a very basic sample. Each threads works with an element 
 * and puts into it its identification 
 *
 */

// Includes
#include <cuda.h>
#include <stdio.h>
#include <cutil_inline.h>

extern "C" void tester();

//Struct
   typedef struct
   {
      int x;
      int y;
      int z;
   } cell;	    

// Variables
cell* h_A;

// Functions
void Cleanup(void);


// Device code
__global__ void FillVec(cell* A, int N)
{
    //Compute array index
    int i = threadIdx.x + blockIdx.x*blockDim.x;
    //Compute result if index is in bounds
    if (i < N)
    {
        A[i].x=threadIdx.x;
        A[i].y=threadIdx.x;
        A[i].z=threadIdx.x;
    }
}

void fill_vectorOnDevice(cell* A_h, int N)
{
    size_t size = N * sizeof(cell);
    cell* d_A;
    // Allocate vector in device memory
    cudaMalloc(&d_A, size);


    // Invoke kernel
    FillVec<<<10, ceil(N/10.0)>>>(d_A, N);

    cutilCheckMsg("kernel launch failure");

    // Copy result from device memory to host memory
    // A_h contains the result in host memory
    cudaMemcpy(A_h, d_A, size, cudaMemcpyDeviceToHost);
    // Free device memory
    cudaFree(d_A);
}


// Host code
void tester()
{

    printf("Vector filling\n");
    int N = 256;
    size_t size = N * sizeof(cell);

    // Allocate input vector h_A 
    h_A = (cell*)malloc(size);
    if (h_A == 0) Cleanup();
    

    // Call the device function
    fill_vectorOnDevice(h_A, N);
    
    // Verify result
    int i;
    for (i = 0; i < N; ++i) {
        printf("%d,%d,%d\n",h_A[i].x,h_A[i].y,h_A[i].z);
    }
    
    Cleanup();
}

void Cleanup(void)
{
    // Free host memory
    if (h_A)
        free(h_A);
        
    cutilSafeCall( cudaThreadExit() );
    
}
